const imageStore: {
  [src: string]:
    | { isResolved: true; image: HTMLImageElement }
    | { isResolved: false; promise: Promise<HTMLImageElement> };
} = {};
export function createImage(src: string): Promise<HTMLImageElement> {
  const cached = imageStore[src];
  if (cached) {
    if (cached.isResolved) return Promise.resolve(cached.image);
    return cached.promise;
  }
  const newCached = (imageStore[src] = {
    isResolved: false,
    promise: new Promise((resolve) => {
      const image = new Image(600, 600);
      image.onload = () => {
        imageStore[src] = { isResolved: true, image: image };
        resolve(image);
      };
      image.src = `images/${src}`;
    }),
  });
  return newCached.promise;
}

export function changeImageColor(
  image: HTMLImageElement,
  newColor: string
): Promise<HTMLImageElement> {
  return new Promise((resolve, reject) => {
    const canvas = document.createElement("canvas");
    canvas.width = 600;
    canvas.height = 600;
    const context = canvas.getContext("2d");
    if (!context) return image;
    context.clearRect(0, 0, 600, 600);
    context.drawImage(image, 0, 0, 600, 600);
    context.fillStyle = newColor;
    context.globalCompositeOperation = "source-in";
    context.fillRect(0, 0, 600, 600);
    canvas.toBlob((blob) => {
      if (!blob) {
        reject();
        return;
      }
      const url = URL.createObjectURL(blob);
      const newImage = new Image();
      newImage.onload = () => {
        URL.revokeObjectURL(url);
        resolve(newImage);
      };
      newImage.src = url;
    });
  });
}
