export const bodyPartBindingDefinitions = {
  eyes: {
    displayName: "eyes",
    bindings: {
      tape: { displayName: "tape" },
    },
    slow: 0.8,
    modifiers: {
      eyes: { ratio: 0.9 },
      arms: { ratio: 0.2 },
      wrists: { ratio: 0.2 },
      hands: { ratio: 0.0 },
    },
  },
  mouth: {
    displayName: "mouth",
    bindings: {
      tape: { displayName: "tape" },
      ballgag: { displayName: "ball-gag" },
      cloth: { displayName: "knotted cloth" },
    },
    slow: 0.99,
    modifiers: {
      eyes: { ratio: 0.9 },
      arms: { ratio: 0.2 },
      wrists: { ratio: 0.2 },
      hands: { ratio: 0.0 },
    },
  },
  arms: {
    displayName: "arms",
    bindings: {
      tape: { displayName: "tape" },
      straps: { displayName: "leather straps" },
    },
    slow: 0.95,
    modifiers: {
      eyes: { ratio: 0.9 },
      wrists: { ratio: 0.8 },
      hands: { ratio: 0.8 },
      legs: { ratio: 0.6 },
    },
  },
  wrists: {
    displayName: "wrists",
    bindings: {
      tape: { displayName: "tape" },
      straps: { displayName: "leather straps" },
    },
    slow: 0.99,
    modifiers: {
      eyes: { ratio: 0.9 },
      arms: { ratio: 0.0 },
      hands: { ratio: 0.0 },
    },
  },
  hands: {
    displayName: "hands",
    bindings: {
      tape: { displayName: "tape" },
      straps: { displayName: "leather mittens" },
    },
    slow: 0.99,
    modifiers: {
      eyes: { ratio: 0.9 },
      mouth: { ratio: 0.5 },
      arms: { ratio: 0.0 },
      wrists: { ratio: 0.8 },
      legs: { ratio: 0.8 },
    },
  },
  legs: {
    displayName: "legs",
    bindings: {
      tape: { displayName: "tape" },
      straps: { displayName: "leather straps" },
    },
    slow: 0.7,
    modifiers: {
      eyes: { ratio: 0.9 },
      arms: { ratio: 0.2 },
      wrists: { ratio: 0.2 },
      hands: { ratio: 0.0 },
    },
  },
  ankles: {
    displayName: "ankles",
    bindings: {
      tape: { displayName: "tape" },
      straps: { displayName: "leather straps" },
    },
    slow: 0.5,
    modifiers: {
      eyes: { ratio: 0.9 },
      arms: { ratio: 0.2 },
      wrists: { ratio: 0.2 },
      hands: { ratio: 0.0 },
      legs: { ratio: 0.0 },
    },
  },
} as const;

export type BodyPartType = keyof typeof bodyPartBindingDefinitions;
export type BindingsAssociationsType = {
  [Part in BodyPartType]: Array<
    keyof typeof bodyPartBindingDefinitions[Part]["bindings"]
  >;
};

export type BindingType<Part extends BodyPartType> =
  BindingsAssociationsType[Part][number];

export const bodyPartList = Object.keys(
  bodyPartBindingDefinitions
) as BodyPartType[];

export function getModifiers(bodyPart: BodyPartType): {
  [key in BodyPartType]?: { ratio: number };
} {
  return bodyPartBindingDefinitions[bodyPart].modifiers;
}
