import { BodyPartType, bodyPartBindingDefinitions, BindingType } from "./definitions";
import { capitalizeText, assertUnreachable } from "./utils";

export function getBodyPartDisplayName(
  bodyPart: BodyPartType,
  options?: { capitalized?: boolean }
): string {
  const displayName = bodyPartBindingDefinitions[bodyPart].displayName;
  const capitalized = !!options?.capitalized;
  if (capitalized) {
    return capitalizeText(displayName);
  }
  return displayName;
}

export function getBindingDisplayName<T extends BodyPartType>(
  bodyPart: T,
  binding: BindingType<T>,
  options?: { capitalized?: boolean }
) {
  const displayName = (() => {
    const castedBodyPart = bodyPart as BodyPartType;
    if (castedBodyPart === "ankles") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else if (castedBodyPart === "arms") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else if (castedBodyPart === "eyes") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else if (castedBodyPart === "hands") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else if (castedBodyPart === "legs") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else if (castedBodyPart === "mouth") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else if (castedBodyPart === "wrists") {
      return bodyPartBindingDefinitions[castedBodyPart].bindings[
        binding as BindingType<typeof castedBodyPart>
      ].displayName;
    } else {
      assertUnreachable(castedBodyPart);
    }
  })();
  const capitalized = !!options?.capitalized;
  if (capitalized) {
    return capitalizeText(displayName);
  }
  return displayName;
}