import { getRandomArrayItem, getRandom, assertUnreachable } from "./utils";

export const effectModifierList = [
  "increaseExplorationLootLevel",
  "decreaseNewBindingsDifficulty",
  "increaseStrugglingStrength",
  "increaseTrapDodge",
] as const;

export type EffectModifierType = typeof effectModifierList[number];
export type EffectModifier = { type: EffectModifierType; multiplier: number };

export type TemporaryEffect = {
  effect: EffectModifier;
  timeout: number;
  maxTimeout: number;
};

export type ActiveEffects = {
  [key in EffectModifierType]: number;
};

export function generateRandomEffect(multiplier: number): EffectModifier {
  return {
    type: getRandomArrayItem(effectModifierList),
    multiplier: 1.0 + multiplier,
  };
}

export function getActiveEffectsFromList(
  effects: EffectModifier[]
): ActiveEffects {
  const result: ActiveEffects = {
    decreaseNewBindingsDifficulty: 1.0,
    increaseExplorationLootLevel: 1.0,
    increaseStrugglingStrength: 1.0,
    increaseTrapDodge: 1.0,
  };
  for (const effect of effects) {
    result[effect.type] *= effect.multiplier;
  }
  return result;
}

export function getEffectDescription(effect: EffectModifier): string {
  if (effect.type === "decreaseNewBindingsDifficulty") {
    return `Decreases the difficulty of new bindings by ${Math.floor(
      100 * (effect.multiplier - 1)
    )}%`;
  }
  if (effect.type === "increaseTrapDodge") {
    return `Increase the probability of dodging traps by ${Math.floor(
      100 * (effect.multiplier - 1)
    )}%`;
  }
  if (effect.type === "increaseExplorationLootLevel") {
    return `Boosts your loot by ${Math.floor(
      100 * (effect.multiplier - 1)
    )}%`;
  }
  if (effect.type === "increaseStrugglingStrength") {
    return `Increases your average struggling strength by ${Math.floor(
      100 * (effect.multiplier - 1)
    )}%`;
  }
  assertUnreachable(effect.type);
}
