import { EffectModifier, generateRandomEffect } from "./effects";
import { getRandomIntegerInRange } from "./utils";

export type PowerUpEffect = {
  duration: number;
  effect: EffectModifier;
};
export type PermanentBonusEffect = { effect: EffectModifier };

export type LootItem =
  | { type: "gem"; value: bigint; cost: bigint }
  | { type: "powerUp"; effect: PowerUpEffect; cost: bigint }
  | { type: "permanentBonus"; effect: PermanentBonusEffect; cost: bigint };

export function generateRandomLootItems(level: number): LootItem[] {
  const powerUpLevel = getRandomIntegerInRange(1, level);
  const permanentBonusLevel =
    getRandomIntegerInRange(1, level) *
    getRandomIntegerInRange(1, Math.ceil(Math.log(1 + level)));

  const powerUpMultiplier = 0.1 * Math.log(1.0 + powerUpLevel);
  const permanentBonusMultiplier = 0.1 * Math.log(1.0 + permanentBonusLevel);
  return [
    {
      type: "gem",
      value: 10n * BigInt(getRandomIntegerInRange(1, level)),
      cost: 0n,
    },
    {
      type: "powerUp",
      effect: {
        duration: 120 * 1000,
        effect: generateRandomEffect(powerUpMultiplier),
      },
      cost: 10n * BigInt(getRandomIntegerInRange(0, powerUpLevel)),
    },
    {
      type: "permanentBonus",
      effect: { effect: generateRandomEffect(permanentBonusMultiplier) },
      cost: 20n * BigInt(permanentBonusLevel),
    },
  ];
}
