import { ImageColors } from "../assets/struggling";
import { CharacterStatus } from "./character";
import {
  bodyPartBindingDefinitions,
  bodyPartList,
  BodyPartType,
} from "./definitions";
import { EffectModifier, TemporaryEffect } from "./effects";
import { LootItem } from "./loot";
import {
  RoomPosition,
  RoomList,
  createStartingArea,
  pickRandomDirection,
} from "./rooms";

export type StruggleEvent = {
  message: string;
  preRandom: number;
  timeout: number;
  maxTimeout: number;
};

export type Store = {
  targetStrugglePart?: BodyPartType;
  character: CharacterStatus;
  map: {
    position: RoomPosition;
    rooms: RoomList;
  };
  mapTargetDirection: "n" | "s" | "e" | "w";
  pendingLoot: {
    choices: Array<{ loot: LootItem[] }>;
  };
  effects: {
    permanent: EffectModifier[];
    temporary: TemporaryEffect[];
  };
  gems: bigint;
  isPaused: boolean;
  showStruggleScreen: boolean;
  showLootChoicePopup: boolean;
  timeout: number;
  difficulty: number;
  appearanceColors?: ImageColors,
  struggleEvents: StruggleEvent[];
  speed: number;
};

export function isSaveFileValid(store: Store): boolean {
  if (!store.pendingLoot) return false;
  for (const bodyPart of bodyPartList) {
    const entry = store.character.bodyParts[bodyPart];
    if (!entry) continue;
    const binding: any = entry.binding;
    const availableBindings: any =
      bodyPartBindingDefinitions[bodyPart].bindings;
    if (!availableBindings[binding]) return false;
  }
  return true;
}

export function createDefaultStore(): Store {
  const map = createStartingArea();
  return {
    character: { bodyParts: {} },
    timeout: 1 * 1000,
    difficulty: 1,
    map: map,
    gems: 0n,
    showStruggleScreen: false,
    showLootChoicePopup: false,
    isPaused: false,
    mapTargetDirection: pickRandomDirection(map.position, map.rooms),
    pendingLoot: {
      choices: [],
    },
    struggleEvents: [],
    effects: {
      permanent: [],
      temporary: [],
    },
    speed: 1,
  };
}
