export function capitalizeText(text: string): string {
  return text
    .split(" ")
    .map((t) =>
      t
        .split("")
        .map((t, i) => (i === 0 ? t.toLocaleUpperCase() : t))
        .join("")
    )
    .join(" ");
}

export function assertUnreachable(value: never): never {
  throw new Error(`Unexpeced type ${value}`);
}

export function getRandom(): number {
  return Math.random();
}

/**
 * @param {number} start The starting value (included in the result)
 * @param {number} end The end value (not included in the result)
 * @returns {number} An integer between start and end, inclusive
 */
export function getRandomIntegerInRange(start: number, end: number): number {
  return Math.floor(start + getRandom() * (end - start));
}

export function getRandomArrayItem<T>(array: readonly T[]): T {
  return array[getRandomIntegerInRange(0, array.length)];
}

export type WeightedArray<T> = Array<{ weight: number; item: T }>;
export type ReadonlyWeightedArray<T> = ReadonlyArray<WeightedArray<T>[number]>;

export function getRandomItemInWeightedArray<T>(
  array: ReadonlyWeightedArray<T>
): T {
  const totalWeight = array.reduce((w, e) => w + e.weight, 0);
  const targetWeight = totalWeight * getRandom();
  let runningWeight = 0;
  for (const entry of array) {
    if (targetWeight < runningWeight + entry.weight) return entry.item;
    runningWeight += entry.weight;
  }
  return array[0].item;
}

export function getCoordinatesFromRandomValue(
  random: number,
  options: { x: { min: number; max: number }; y: { min: number; max: number } }
): { x: number; y: number } {
  const xRange = options.x.max - options.x.min;
  const xBase = Math.floor(random * xRange);
  const xCoord = options.x.min + xBase;
  const randomRemainder = random * xRange - xBase;
  const yRange = options.y.max - options.y.min;
  const yCoord = options.y.min + Math.floor(randomRemainder * yRange);
  return { x: xCoord, y: yCoord };
}
